/*
 * Copyright 2022 KylinSoft Co., Ltd.
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef DESKTOPBACKGROUND_H
#define DESKTOPBACKGROUND_H

#include <QQuickItem>
#include <QQuickPaintedItem>
#include <QImage>

class QGSettings;
class QColor;

/*!
 * \brief   背景绘制的Item，导出由QML使用进行背景的绘制
 * \details 方便前端进行背景绘制，并提供虚化、背景图片拉伸的能力
 */
class DesktopBackground : public QQuickPaintedItem
{
    Q_OBJECT
    Q_PROPERTY(qreal blurAlpha READ blurAlpha WRITE setBlurAlpha NOTIFY blurAlphaChanged)
public:
    explicit DesktopBackground(QQuickPaintedItem *parent = nullptr);
    ~DesktopBackground();
    void paint(QPainter *painter);
    qreal blurAlpha() { return m_blurAlpha; }
    void setBlurAlpha(qreal alpha);
    QRect scaledRect(const QImage &image);
    QRect getDestRect(const QImage &image, const QRectF &desktopRect);

signals:
    void loadPicture(const QString &fileName);
    void blurAlphaChanged();

private slots:
    void onBlurAlphaChanged();

private:
    void init();
    QString getBackgroundFile();
    QString getBackgroundFileFromAccount();
    QString getBackgroundFillStyle();
    double getSystemScalingFactor();
    void updateBlurBackground();
    int edge(int x, int i, int w);
    float gaussFunc1D(int x, float sigma);
    void getKernal(int radius, float* weight);
    void bluredImage(int blur_radius, QImage &blurImage);

private:
    QGSettings *m_backgroundSettings = nullptr;
    QString m_backgroundFile;
    QString m_backgroundFillStyle = "scaled";
    double m_scalingFactor = 1.5;
    static QImage m_blurBackground;
    static QImage m_nonBlurBackground;
    qreal m_blurAlpha;
    static quint32 m_blurRadius;
    bool m_usePureColor = false;
    QColor m_pureColor = Qt::black;
};

#endif // DESKTOPBACKGROUND_H
