/*
 * Copyright 2024 KylinSoft Co., Ltd.
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef MultitaskViewModelHandler_H
#define MultitaskViewModelHandler_H

#include <QObject>

class MultitaskViewModelHandler : public QObject
{
    Q_OBJECT
    Q_DISABLE_COPY(MultitaskViewModelHandler)

public:
    explicit MultitaskViewModelHandler() = default;
    ~MultitaskViewModelHandler() = default;

    virtual void removeAppWindow(QString windowId) = 0;
    virtual void moveAppWindowToDesktop(QString windowId, int srcDesktopInex, int destDesktopInex) = 0;
    virtual void activateAppWindow(QString windowId) = 0;
    virtual QIcon getWindowIcon(QString windowId) = 0;
    virtual void removeDesktop(int desktopIndex) = 0;
    virtual void appendDesktop() = 0;
    virtual void moveDesktop(int srcDesktopIndex, int destDesktopIndex) = 0;
    virtual int getCurrentDesktop() = 0;
    virtual bool isCurrentDesktopShowingDesktop() = 0;
    virtual void setCurrentDesktop(int desktopIndex) = 0;
    virtual QRect getFullScreenGeometry(int desktopIndex, int screenIndex) =0;
    virtual bool hasModalChildWindow(QString windowId) = 0;
    virtual bool isExtensionMode() = 0;
    virtual int logicalScreenCount() = 0;
    virtual QList<QVariant> createWindowList(int desktopIndex, int screenIndex) = 0;
    virtual int getNumberOfDesktops() = 0;
    virtual QString getDesktopUuid(int desktopIndex) = 0;
    virtual QString getOutputUuid(int outputIndex) = 0;

signals:
    void appWindowCreated();
    void appWindowRemoved(QString windowId);
    void appWindowActivated();
    void appWindowDesktopChanged(QString windowId, int oldDesktopIndex,int newDesktopIndex);
    void desktopRemoved(int desktopIndex);
    void desktopAppended(int desktopIndex);
    void desktopMoved(int oldDesktopIndex, int newDesktopIndex);
    void currentDesktopChanged(int oldCurrentDesktopIndex, int newCurrentDesktopIndex);
    void screenCountChanged();
    void screenSizeChanged();
    void desktopNumberChanged();
    void tabletModeChanged(bool tabletMode);

};


#endif // MultitaskViewModelHandler_H
