/*
 * Copyright 2022 KylinSoft Co., Ltd.
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

import QtQuick 2.12
import QtQuick.Controls 2.5
import QtQuick.Layouts 1.3
import QtGraphicalEffects 1.0

Rectangle {
    id : root
    visible : false

    property string windowType : "none"
    property var appCloseButtonDefaultColor : Qt.rgba(1, 1, 1, 0.15)
    property var appCloseButtonDefaultImage : "qrc:/images/application-exit-symbolic-white.svg"
    property var buttonStateList : ["default", "hover", "click"];
    state : "default"
    property alias closeButtonContainMouse: closeButtonMouseArea.containsMouse
    property var imageSource : "qrc:/images/application-exit-symbolic.svg"

    signal entered()
    signal exited()
    signal pressed()
    signal clicked()

    states : [
        State {
            name : "default"
            PropertyChanges {
                target : root
                color : windowType == "desktop" ?  desktopCloseButtonDefaultColor : appCloseButtonDefaultColor
                imageSource : windowType == "desktop" ?  closeButtonGeneralImage : appCloseButtonDefaultImage
            }
        },
        State {
            name : "hover"
            PropertyChanges {
                target : root
                color : closeButtonGeneralHoverColor
                imageSource : closeButtonGeneralImage
            }
        },
        State {
            name : "click"
            PropertyChanges {
                target : root
                color : closeButtonGeneralClickColor
                imageSource : closeButtonGeneralImage
            }
        }
    ]

    Image {
        id: closeWindow
        anchors.centerIn : parent
        source : imageSource
        sourceSize.width: Math.floor(parent.width / 2)
        sourceSize.height: Math.floor(parent.width / 2)
    }

    MouseArea {
        id : closeButtonMouseArea
        anchors.fill : parent
        hoverEnabled : true

        onEntered: {
            root.state = "hover"
            root.entered()
        }
        onExited: {
            root.exited()
        }
        onPressed: {
            root.state = "press"
            root.pressed()
        }
        onClicked: {
            root.clicked()
        }
    }
}
