/*
 * Copyright 2022 KylinSoft Co., Ltd.
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

import QtQuick 2.12
import QtQuick.Controls 2.5
import QtQuick.Layouts 1.3
import QtQuick.Window 2.12
import QtGraphicalEffects 1.0
import "Controller.js" as Controller

Item {
    id : root
    width : isVerticalLayout ? desktopRectWidth : desktopRect.width
    height : isVerticalLayout ? desktopRect.height : desktopRectHeight
    activeFocusOnTab: true
    focus : true
    visible : desktopListView.count < 16 ? true  : false
    property string windowType : "creation"
    property var newWorkspaceController : new Controller.CreationController(multitaskViewModel, root, multitaskViewManager, ["app","desktop"]);

    onActiveFocusChanged: {
        if (root.activeFocus) {
            desktopRect.desktopTabBorderVisble = true
            desktopListView.currentIndex = desktopWindowModel.count - 1
            if (isVerticalLayout && desktopListView.contentX + desktopListView.contentXoffset < desktopListView.contentWidth - desktopListView.width) {
                contentMoveAnimation.to = isLayoutLeftToRight ? desktopListView.contentWidth - desktopListView.width : -desktopListView.contentWidth
                contentMoveAnimation.start()
            }
            if (!isVerticalLayout && desktopListView.contentY < desktopListView.contentHeight - desktopListView.height) {
                contentMoveAnimation.to = desktopListView.contentHeight + listviewTopMargin - desktopListView.height
                contentMoveAnimation.start()
            }
            multitaskViewManager.activeAreaChangeDataCollect("New Workspace Button Area")
        }
    }

    Keys.onPressed: {
        if (event.key === Qt.Key_Return || event.key === Qt.Key_Enter || event.key === Qt.Key_Space)
        {
            newWorkspaceController.onWindowClicked()
            event.accepted = true
        } else if (event.key === Qt.Key_Escape) {
            newWorkspaceController.closeMultitaskView()
        } else if (event.modifiers & Qt.MetaModifier) {
            if (event.key === Qt.Key_Tab) {
                newWorkspaceController.closeMultitaskView()
            }
        } else if (event.key === Qt.Key_Tab) {
            if (appWindowModel.count === 0) {
                desktopListView.currentIndex = 0
            }
        }
    }

    states: State {
        name: "active"
        when: root.activeFocus && (multitaskViewManager.shouldShowTabBorder() || desktopRect.desktopTabBorderVisble)
        PropertyChanges { target: createDesktopButtonBorder; border.color : tabFocusBorderColor}
    }

    Item {
        width : desktopRectWidth
        height : mainVerticalLayout.height
        anchors.centerIn : parent

        MouseArea {
            id : previewMouseArea
            anchors.fill : parent
            preventStealing : true
            onClicked : {
                newWorkspaceController.onWindowClicked()
            }
        }

        ColumnLayout {
            id : mainVerticalLayout
            anchors.centerIn : parent
            spacing : spacingWidth
            Layout.fillHeight : true
            width : parent.width

            Label {
                id : caption
                verticalAlignment: Qt.AlignVCenter
                horizontalAlignment: isLayoutLeftToRight ? Qt.AlignLeft : Qt.AlignRight
                elide : isLayoutLeftToRight ? Text.ElideRight : Text.ElideLeft
                Layout.fillWidth : true
                Layout.leftMargin : workspaceBorderWidth
                height : desktopCaptionHeight
                text : qsTr("New Desktop")
                font.pointSize: multitaskViewModel.getUKUIFontSize()
                font.family : multitaskViewModel.getUKUIFont()
                color: rootWindow.desktopCaptionColor
                ToolTip.text: caption.text
                ToolTip.delay: 400
                ToolTip.visible: captionArea.containsMouse && caption.truncated
                MouseArea {
                    id: captionArea
                    anchors.fill: parent
                    hoverEnabled: true
                }
            }

            Rectangle {
                id : createDesktopButtonBorder
                width : desktopRectWidth
                height : desktopRectHeight - desktopCaptionHeight
                color : "transparent"
                border.width : workspaceBorderWidth
                border.color : "transparent"
                radius : workspaceRadius !== 0 ? workspaceRadius + workspaceBorderWidth : 0
                
                Rectangle {
                    id : createDesktopButton
                    width : desktopThumbnailWidth
                    height : desktopThumbnailHeight
                    anchors.centerIn : parent
                    radius : workspaceRadius
                    color : rootWindow.createDesktopButtonDefaultColor

                    Rectangle {
                        anchors.centerIn : parent
                        color : rootWindow.desktopCaptionColor
                        width : Math.floor(24 * scaleFactor)
                        height : Math.floor(2 * scaleFactor)
                    }
                    Rectangle {
                        anchors.centerIn : parent
                        color : rootWindow.desktopCaptionColor
                        width : Math.floor(2 * scaleFactor)
                        height : Math.floor(24 * scaleFactor)
                    }

                    DropArea {
                        id : newDesktopDropArea
                        anchors.fill : parent
                        onEntered : {
                            createDesktopButton.color = rootWindow.createDesktopButtonReceiveColor
                        }
                        onExited : {
                            createDesktopButton.color = rootWindow.createDesktopButtonDefaultColor
                        }
                        onDropped : {
                            createDesktopButton.color = rootWindow.createDesktopButtonDefaultColor
                            newWorkspaceController.onSourceDropped(drop.source, rootWindow.highlightDesktop, desktopWindowModel.count);
                            rootWindow.updateDesktopWindowModel()
                        }
                    }
                }
            }
        }//culumnLayout
    }
}
