/*
 * Copyright 2022 KylinSoft Co., Ltd.
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

import QtQuick 2.12
import QtQuick.Controls 2.5
import QtQuick.Layouts 1.3
import QtGraphicalEffects 1.0

Rectangle {
    id : root
    visible : false
    width : iconAndCloseButtonHeight
    height : width
    radius : 14
    color : "transparent"
    state : "default"
    property var imageSource : "qrc:/images/ukui-taskview-close-default.svg"
    property alias closeButtonContainMouse: closeButtonMouseArea.containsMouse

    signal entered()
    signal exited()
    signal pressed()
    signal clicked()

    states :  [
        State {
            name : "default"
            PropertyChanges { target : root; imageSource : "qrc:/images/ukui-taskview-close-default.svg" }
        },
        State {
            name : "hover"
            PropertyChanges { target : root; imageSource : "qrc:/images/ukui-taskview-close-hover.svg" }
        },
        State {
            name : "press"
            PropertyChanges { target : root; imageSource : "qrc:/images/ukui-taskview-close-click.svg" }
        }
    ]

    Image {
        id: closeWindow
        anchors.fill : parent
        source : imageSource
        sourceSize.width: parent.width
        sourceSize.height: parent.height
    }

    MouseArea{
        id : closeButtonMouseArea
        anchors.fill : parent
        hoverEnabled : true
        onEntered: {
            root.state = "hover"
            root.entered()
        }

        onExited: {
            root.exited()
        }

        onPressed: {
            root.state = "press"
            root.pressed()
        }

        onClicked: {
            root.clicked()
        }
    }
}
