/*
 * Copyright 2022 KylinSoft Co., Ltd.
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

import QtQuick 2.12
import QtQuick.Controls 2.5
import QtQuick.Layouts 1.3
import QtQuick.Window 2.12
import QtGraphicalEffects 1.0
import org.kylin.background 3.0 as KWin

Repeater{
    id : allrootRepeater
    model : screencnt
    property int screencnt : multitaskViewModel.logicalScreenCount
    //水平布局方向
    property bool isLayoutLeftToRight: multitaskViewManager.isLayoutLeftToRight()

    Item {
        id : tabletViewMain
        property var isWlcom : multitaskViewManager.isWlcom()
        property var screenRect : multitaskViewModel.getFullScreenGeometry(multitaskViewModel.getCurrentDesktop(), index)
        // 整体缩放比
        property var tabletScaleFactor : screenRect.width > screenRect.height ? screenRect.height / 1080 : screenRect.width / 1080
        // 预览窗口宽高比
        property var previewWinAspectRatio: screenRect.width / screenRect.height
        // 图标和缩略图的间距
        property var iconToThumbSpacing : Math.floor(18 * tabletScaleFactor)
        // 图标高度
        property var iconAndCloseButtonHeight : Math.floor(44 * tabletScaleFactor)
        // 缩略图高度
        property var preWinHeight
        // 缩略图宽度
        property var preWinWidth
        property var borderWidth : Math.floor(4 * tabletScaleFactor)
        // 预览窗口列间距
        property var preWinVecSpace
        // 预览窗口行间距
        property var preWinHorSpace
        // 滚动区域距离左侧和右侧的间距
        property var scrollViewLeftSpace
        // 滚动区域高度
        property var scrollViewHeight
        // 滚动区域 y 轴起始高度
        property var scrollViewYStartPoint : getCurrentPreWindowHeight()
        property ListModel tabletViewRootModel : ListModel{}
        property var language : multitaskViewModel.getCurrentLanguage()

        x : Math.floor(screenRect.x)
        y : Math.floor(screenRect.y)
        width : Math.floor(screenRect.width)
        height : Math.floor(screenRect.height)

        // 这里放置后台提供的桌面背景的 Item
        KWin.DesktopBackground {
            anchors.fill : parent
            blurAlpha : 0.7
        }

        function getCurrentPreWindowHeight()
        {
            console.log("当前开始计算每一个预览窗口的尺寸")
            if ( screenRect.width > screenRect.height )
            {
                scrollViewYStartPoint = Math.floor(99 * tabletScaleFactor)
                preWinHeight = Math.floor(320 * tabletScaleFactor)
                preWinWidth = Math.floor(preWinHeight * previewWinAspectRatio)
                preWinVecSpace = Math.floor(112 * tabletScaleFactor)
                preWinHorSpace = Math.floor(56 * tabletScaleFactor)
                scrollViewLeftSpace = Math.floor(112 * tabletScaleFactor)
                scrollViewHeight = Math.floor(820 * tabletScaleFactor)
            } else {
                scrollViewYStartPoint = Math.floor(196 * tabletScaleFactor)
                preWinHeight = Math.floor(498 * tabletScaleFactor)
                preWinWidth = Math.floor(preWinHeight * previewWinAspectRatio)
                preWinVecSpace = Math.floor(96 * tabletScaleFactor)
                preWinHorSpace = Math.floor(68 * tabletScaleFactor)
                scrollViewLeftSpace = Math.floor(96 * tabletScaleFactor)
                scrollViewHeight = Math.floor(1196 * tabletScaleFactor)
            }
        }
        focus : true
        property var loadedWindowNum : 0
        /*
            此处的focus不可以删除，因为后台在grabbedKeyboardEvent()函数中增加了setfocus()
            不增加focus:true前端无法接受键盘事件
            这是正向解决方法，其他方法非正向解决，不推荐！！！
        */

        signal blankAreaClickSignal()
        signal clearAllWindowButtonClickSignal()
        //由于执行小窗口关闭动效时，会关闭多任务视图多次，导致崩溃，故增加此属性
        property bool alreadyCloseView: false

        function sequentialLoadWindow() {
            var appWindowList =  multitaskViewModel.getAppWindowList(multitaskViewModel.getCurrentDesktop(), tabletViewMain.index)
            var appWindowCount = appWindowList.length
            if (loadedWindowNum < appWindowCount && loadedWindowNum < 6 ) {
                var info = appWindowList[loadedWindowNum]
                tabletViewRootModel.append({"x": info.x, "y": info.y,
                                            "width": info.width, "height": info.height,
                                            "id": info.id, "minimized": info.minimized,
                                            "caption": info.caption, "uuid": String(info.uuid)});
                loadedWindowNum = loadedWindowNum + 1
                // console.log("加载窗口中，目前已加载窗口数量为" + loadedWindowNum)

            } else {
                sequentialLoadWindowTimer.stop()
                // console.log("计时器停止，窗口数量为： " + tabletViewRootModel.count)
            }
            if (loadedWindowNum >= 6) {
                sequentialLoadWindowTimer.stop()
                console.log("计时器停止，接下来一次性加载所有窗口")
                while (loadedWindowNum < appWindowCount) {
                    var info = appWindowList[loadedWindowNum]
                    tabletViewRootModel.append({"x": info.x, "y": info.y,
                                                "width": info.width, "height": info.height,
                                                "id": info.id, "minimized": info.minimized,
                                                "caption": info.caption, "uuid": String(info.uuid)});
                    loadedWindowNum = loadedWindowNum + 1
                }
            }
            if (loadedWindowNum >= appWindowCount) {
                sequentialLoadWindowTimer.stop()
                // console.log("所有窗口加载完毕，显示全部清除按钮")
                clearAllWindowButtonAppearAni.start()
            }
        }

        function closeMultitaskView() {
            if (!alreadyCloseView) {
                alreadyCloseView = true
                console.log("closeMultitaskView() !")
                multitaskViewManager.close()
            }
        }

        // 控制启动动效中，小窗口之间的间隔
        Timer {
            id : sequentialLoadWindowTimer
            interval : 150
            repeat : true
            running : false
            triggeredOnStart : false
            onTriggered: {
                // console.log("计时器启动 间隔150")
                tabletViewMain.sequentialLoadWindow()
            }
        }

        Rectangle {
            id : scrollBackgroundArea
            anchors.fill: parent
            color : Qt.rgba(0, 0, 0, 0.35)
            // 无应用时的窗口显示
            Text {
                id : noWindowText
                anchors.centerIn : parent
                font.bold : true
                font.pixelSize : Math.floor(25 * tabletScaleFactor)
                color : "white"
                text : qsTr("No recent tasks") // 无应用开启
                visible : tabletViewRootModel.count == 0
                opacity : 0

                SequentialAnimation{
                    id :  noWindowTipsAnimation
                    // 无窗口文字出现动画
                    NumberAnimation {
                        id : noWindowTipsAppearAni
                        target : noWindowText
                        property : "opacity"
                        duration : 1000
                        easing.type : Easing.InOutQuart
                        from : 0
                        to : 1
                    }
                    // 无窗口文字消失动画
                    NumberAnimation {
                        id : noWindowTipsDisappearAni
                        target : noWindowText
                        property : "opacity"
                        duration : 1000
                        easing.type : Easing.InOutQuart
                        from : 1
                        to : 0
                    }
                    onFinished: {
                        console.log("无窗口文字消失动画")
                        tabletViewMain.closeMultitaskView()
                    }
                }
            }
        }

        // 预览窗口显示区域
        ScrollView {
            id : previewWindowShownRegion
            anchors.fill : parent

            //更新滚动区域的大小，之所以这么做，是为了解决滚动区域增加鼠标事件后，滚动失效的问题
            function getContentWidth()
            {
                var maxColumn = Math.ceil(tabletViewRootModel.count / 2)
                var scrollViewContentWidth = parent.width
                if (tabletViewRootModel.count <= 4)
                {
                    scrollViewContentWidth = parent.width
                    return Math.floor(scrollViewContentWidth)
                }
                scrollViewContentWidth = maxColumn * preWinWidth + maxColumn * gridLayout.columnSpacing + 2 * gridLayout.leftPadding
                return Math.floor(scrollViewContentWidth)
            }
            contentWidth : Math.floor(previewWindowShownRegion.getContentWidth())
            contentHeight : Math.floor(this.height)
            ScrollBar.horizontal.policy : ScrollBar.AlwaysOff
            ScrollBar.vertical.policy : ScrollBar.AlwaysOff

            MouseArea {
                anchors.fill : parent
                onClicked : {
                    if (multitaskViewModel.isTabletMode) {
                        multitaskViewManager.setShowingDesktop(true)
                    }
                    tabletViewMain.blankAreaClickSignal()
                }
            }

            Grid {
                id : gridLayout
                anchors.fill : parent
                anchors.centerIn : parent
                topPadding : Math.floor(scrollViewYStartPoint)
                columnSpacing : Math.floor(preWinVecSpace)
                rowSpacing : Math.floor(preWinHorSpace)
                leftPadding : Math.floor(scrollViewLeftSpace)
                rightPadding : Math.floor(scrollViewLeftSpace)
                rows : 2
                flow : Grid.TopToBottom
                transformOrigin : "TopLeft"

                //显示多任务视图时，每个Item在显示的时候执行的动效
                add : Transition {
                    id : displayPreviewWinAnimation
                    ParallelAnimation {
                        NumberAnimation {
                            property : "opacity"
                            duration : 900
                            easing.type : Easing.InOutQuart
                            from : 0
                            to : 1
                        }
                        NumberAnimation {
                            property : "x"
                            duration : 900
                            easing.type : Easing.InOutQuart
                            from : Math.floor(parent.width / 2 - 510/*窗口的宽度*/ / 2)
                        }
                        NumberAnimation {
                            property : "y"
                            duration : 900
                            easing.type : Easing.InOutQuart
                            from : Math.floor(parent.height)
                        }
                        NumberAnimation {
                            property : "scale"
                            from : 0
                            to : 1.0
                            duration : 450
                        }
                    }
                }

                //关闭小窗口时，W补齐方式的动画
                move : Transition {
                    NumberAnimation{
                        properties : "x,y";
                        easing.type : Easing.InOutQuart;
                        duration : 300
                    }
                }

                Repeater {
                    id : previewWindowRepeater
                    model : tabletViewRootModel

                    Item {
                        id : appWindowBack
                        height : Math.floor(preWinHeight + iconToThumbSpacing + iconAndCloseButtonHeight)
                        width : Math.floor(preWinWidth) + borderWidth * 2

                        TabletPreviewWindow {
                            id : appWindow
                            height : parent.height
                            width : parent.width
                            x_wid : model.id
                            wid : model.uuid
                            previewWindowModel : tabletViewRootModel
                            screenIndex : tabletViewMain.index
                            captionText : model.caption
                            preViewNoScaledWidth : model.width
                            preViewNoScaledHeight : model.height
                            preViewNoScaledXCoordinate : model.x
                            preViewNoScaledYCoordinate : model.y
                        }
                    }
                }
            }
        }

        // 清除所有窗口的按钮
        Rectangle {
            id : clearAllWindowButton
            property var bottomMargin : Math.floor(38 * tabletScaleFactor)
            height : Math.floor(48 * tabletScaleFactor)
            width : language === "Chinese" ? Math.floor(126 * tabletScaleFactor) : Math.floor(184 * tabletScaleFactor)
            color :  Qt.rgba(1, 1, 1, 0.4)
            radius : Math.floor(10 * tabletScaleFactor)
            anchors.verticalCenter : parent.verticalCenter
            anchors.verticalCenterOffset : Math.floor(tabletViewMain.height / 2) - bottomMargin - this.height
            anchors.horizontalCenter : parent.horizontalCenter
            visible : tabletViewRootModel.count > 0
            opacity : 0

            Image {
                id : deleteImage
                width : Math.floor(24 * tabletScaleFactor)
                height : Math.floor(24 * tabletScaleFactor)
                x : Math.floor(27 * tabletScaleFactor)
                anchors.verticalCenter : parent.verticalCenter
                source : "qrc:/images/tablet_delete.svg"
            }

            Label{
                id : deletText
                x : Math.floor(60 * tabletScaleFactor)
                anchors.verticalCenter : parent.verticalCenter
                Layout.fillWidth : true
                text : qsTr("Clear") // 清除
                elide : Text.ElideRight
                font.pixelSize : Math.floor(18 * tabletScaleFactor)
                color : "#000000"
            }

            MouseArea {
                anchors.fill : parent
                hoverEnabled : true
                onEntered: {
                    clearAllWindowButton.color =  Qt.rgba(1, 1, 1, 0.6)
                }
                onPressed: {
                    clearAllWindowButton.color =  Qt.rgba(1, 1, 1, 0.8)
                }
                onExited: {
                    clearAllWindowButton.color =  Qt.rgba(1, 1, 1, 0.4)
                }
                onClicked: {
                    console.log("全部清除被点击！")
                    clearAllWindowButtonDisappearAni.start()
                    tabletViewMain.clearAllWindowButtonClickSignal()
                }
            }

            //全部清除按钮出现的动画
            NumberAnimation {
                id : clearAllWindowButtonAppearAni
                target : clearAllWindowButton
                properties : "opacity"
                from : 0
                to : 1
                duration : tabletViewRootModel.count >= 6 ? 5 * 100 + 300 : (tabletViewRootModel.count - 1) * 100 + 300
                easing.type : Easing.InOutQuart
            }

            //全部清除按钮消失的动画
            NumberAnimation {
                id : clearAllWindowButtonDisappearAni
                target : clearAllWindowButton
                properties : "opacity"
                from : 1
                to : 0
                duration : 300
                easing.type : Easing.InOutQuart
            }

            Connections {
                target : tabletViewMain
                function onBlankAreaClickSignal() {
                    clearAllWindowButtonDisappearAni.start()
                }
            }
        }

        Component.onCompleted: {
            if (multitaskViewModel.currentDesktopHasWindow()) {
                console.log("顺序加载窗口的启动计时器")
                sequentialLoadWindowTimer.start()
            } else {
                console.log("启动无窗口动画")
                noWindowTipsAnimation.start()
            }
        }

        Keys.onPressed: {
            if (event.key === Qt.Key_Escape) {
                closeMultitaskView();
                return;
            }
            if (event.modifiers & Qt.MetaModifier) {
                if (event.key === Qt.Key_Tab) {
                    closeMultitaskView();
                    return;
                }
            }
        }

        Timer {
            id : closeViewTimer
            interval : 0
            repeat : false
            running : false
            triggeredOnStart : false
            onTriggered: {
                closeMultitaskView()
            }
        }

        Connections {
            target: multitaskViewModel
            function onAppWindowRemoved(windowId) {
                for(var i = 0; i < tabletViewRootModel.count; i++){
                    var info = tabletViewRootModel.get(i);
                    if(info.uuid === windowId) {
                        tabletViewRootModel.remove(i);
                        if (!multitaskViewModel.currentDesktopHasWindow()) {
                            closeViewTimer.start()
                        }
                        return;
                    }
                }
            }
        }
    }
}
