#ifndef MULTITASKVIEWMODELBYCOMPOSITEHANDLER_H
#define MULTITASKVIEWMODELBYCOMPOSITEHANDLER_H
#include "../multitaskviewmodelhandler.h"
#include "../multitaskviewmodel.h"
#include <QObject>
#include <QIcon>
#include <QList>
#include <QMap>
#include <QRect>
#include <QVariantList>
#include <QString>
#include <kwineffects.h>

class MultitaskViewModelByCompositeHandler : public MultitaskViewModelHandler
{
    Q_OBJECT

public:
    explicit MultitaskViewModelByCompositeHandler(QObject *parent = nullptr);
    ~MultitaskViewModelByCompositeHandler() = default;

    void removeAppWindow(QString windowId) override;
    void moveAppWindowToDesktop(QString windowId, int srcDesktopInex, int destDesktopInex) override;
    void activateAppWindow(QString windowId) override;
    QIcon getWindowIcon(QString windowId) override;
    void removeDesktop(int desktopIndex) override;
    void moveDesktop(int srcDesktopIndex, int destDesktopIndex) override;
    int getNumberOfDesktops() override;
    void appendDesktop() override;
    int getCurrentDesktop() override;
    bool isCurrentDesktopShowingDesktop() override;
    void setCurrentDesktop(int desktopIndex) override;
    QRect getFullScreenGeometry(int desktopIndex, int screenIndex) override;
    bool hasModalChildWindow(QString windowId) override;
    bool isExtensionMode() override;
    int logicalScreenCount() override;
    QList<QVariant> createWindowList(int desktopIndex, int screenIndex) override;
    QString getDesktopUuid(int desktopIndex) override;
    QString getOutputUuid(int outputIndex) override;
private:
    static WId getWindowId(KWin::EffectWindow* window);
    static QUuid getUuid(KWin::EffectWindow *window);
    bool shouldAcceptWindow(KWin::EffectWindow* window);
    void connectSignals();

private slots:
    void onWindowAdded(KWin::EffectWindow* window);
    void onWindowDeleted(KWin::EffectWindow* window);
    void onWindowActivated(KWin::EffectWindow* window);
    void onDesktopPresenceChanged(KWin::EffectWindow* window, int oldDesktopIndex, int newDesktopIndex);
    void onNumberScreensChanged();

};


#endif // MULTITASKVIEWMODELBYCOMPOSITEHANDLER_h
